BEGIN \
{
  previousAddress = "";
  previousSymbol = "";
  previousKind = "";
}

#-----

/; Area: _CODE_/ \
{
  # this introduces a banked code page ==>
  # extract bank number as prefix for following labels
  Output_flush();
  thisLine = $0;
  partCount = split(thisLine, parts, "_");
  bankPrefix = "0" parts[partCount];
  print;
  next;
}

#-----

/; Area: / \
{
  # this introduces some nonbanked code page or other area
  # ==> bank number for following labels is empty
  Output_flush();
  bankPrefix = "";
  print;
  next;
}

#-----

/ G\$/ \
{
  # global symbol definition ==> skip
  next;
}

#-----

/ XG\$/ || / XF/ || /_end/ \
{
  # an exit from a function or end of a global symbol definition
  thisLine = $0;
  currentAddress = $1;
  adjustCurrentAddress();

  if (thisLine ~ /_end$/) {
    symbol = $2;
    sub(/_end$/, "", symbol);
  } else {
    split(thisLine, parts, "$");
    symbol = parts[2];
  }

  symbol = "END_" symbol;
  Output_write("end", currentAddress, symbol);
  next;
}

#-----

/ F[^ \$]+\$/ || /_start$/ \
{
  # a start of a routine definition
  thisLine = $0;
  currentAddress = $1;
  adjustCurrentAddress();

  if (thisLine ~ /_start$/) {
    symbol = $2;
    sub(/_start$/, "", symbol);
  } else {
    split(thisLine, parts, "$");
    symbol = parts[2];
  }

  Output_write("enter", currentAddress, symbol);
  next;
}

#-----

/ C\$/	\
{ 
  thisLine = $0;
  currentAddress = $1;
  adjustCurrentAddress();
  partCount = split(thisLine, parts, "$");
  symbol = "line_" parts[3] "(" parts[2] ")";
  Output_write("line", currentAddress, symbol);
  next;
}

#-----

/_st[^_]*$/ \
{
  # string definition ==> skip
  next;
}

#-----

/.*/ \
{
  thisLine = $0;
  currentAddress = $1;
  symbol = $2;
  adjustCurrentAddress();
  Output_write("other", currentAddress, symbol);
}

#-----

END \
{
  Output_flush();
}

#--------------------

function adjustCurrentAddress() \
{
  # adjusts <currentAddress> when in banked code area
  if (bankPrefix != "") {
    split(currentAddress, parts, ":");
    currentAddress = bankPrefix ":" parts[2];
  }
}

#--------------------

function Output_flush () \
{
  if (previousSymbol != "") {
    print previousAddress " " previousSymbol;
    previousSymbol = "";
  }
}

#--------------------

function Output_write(kind, currentAddress, symbol) \
{
  # is some unprocessed line left?
  if (previousAddress != currentAddress && previousSymbol != "") {
    Output_flush();
  }

  # DEBUG
  #if (currentAddress >= "01:586C" && currentAddress <= "01:592C") {
  #  print ">>L: " thisLine;
  #  print ">>P: " kind "/" currentAddress "/" symbol;
  #}

  if (previousAddress != currentAddress || kind == "enter" || kind == "section" \
     || previousKind != "enter" && kind == "end") {
    previousSymbol = symbol;
    previousKind = kind;
  }

  previousAddress = currentAddress;
}
